package com.yiqi.core.mapper;

import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.yiqi.core.domain.dto.GoodsUpdateSalesDTO;
import com.yiqi.core.domain.dto.weapp.WeAppGoodsInfoQueryDTO;
import com.yiqi.core.domain.vo.GoodsInfoQueryVO;
import com.yiqi.core.domain.vo.weapp.WeAppGoodsInfoQueryVO;
import com.yiqi.order.domain.dto.RetailQueryGoodsDTO;
import com.yiqi.order.domain.vo.OrderGoodsSkuInfoVO;
import org.apache.ibatis.annotations.Mapper;
import com.yiqi.core.domain.GoodsInfo;
import org.apache.ibatis.annotations.Param;

/**
 * 商品管理Mapper接口
 *
 * @author ruoyi
 * @date 2023-08-21
 */
public interface GoodsInfoMapper extends BaseMapper<GoodsInfo> {

    public List<WeAppGoodsInfoQueryVO> findPage(WeAppGoodsInfoQueryDTO weAppGoodsInfoQueryDTO);

    /**
     * 查询商品管理
     *
     * @param id 商品管理主键
     * @return 商品管理
     */
    public GoodsInfo selectGoodsById(Long id);

    /**
     * 查询商品管理列表
     *
     * @param goodsInfoQueryVO 商品管理
     * @return 商品管理集合
     */
    public List<GoodsInfoQueryVO> selectGoodsList(GoodsInfoQueryVO goodsInfoQueryVO);

    /**
     * 新增商品管理
     *
     * @param goods 商品管理
     * @return 结果
     */
    public int insertGoods(GoodsInfo goods);

    /**
     * 修改商品管理
     *
     * @param goods 商品管理
     * @return 结果
     */
    public int updateGoods(GoodsInfo goods);

    /**
     * 删除商品管理
     *
     * @param id 商品管理主键
     * @return 结果
     */
    public int deleteGoodsById(Long id);

    /**
     * 批量删除商品管理
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteGoodsByIds(Long[] ids);

    /**
     * 查询商品管理
     *
     * @param ids 商品管理主键
     * @return 商品管理
     */
    public int selectGoodsByBrandIds(Long[] ids);

    /**
     * 查询商品管理
     *
     * @param ids 商品管理主键
     * @return 商品管理
     */
    public int selectGoodsCategoryIds(Long[] ids);

    /**
     * 通过分类+门店id，查询零售商品
     *
     * @param retailQueryGoodsDTO 商品分类id
     * @return 商品管理
     */
    public List<GoodsInfo> selectGoodsByCategoryId(RetailQueryGoodsDTO retailQueryGoodsDTO);

    public int addGoodsSaleNumber(List<GoodsUpdateSalesDTO> goodsUpdateSales);

    public int minusGoodsSaleNumber(List<GoodsUpdateSalesDTO> goodsUpdateSales);

    public List<OrderGoodsSkuInfoVO> getSkuInfoListByApp(@Param("skuIdList")List<Long> skuIdList);

    public List<OrderGoodsSkuInfoVO> getSkuInfoListByStore(@Param("skuIdList")List<Long> skuIdList,@Param("sourceType") String sourceType,@Param("orgId") Long orgId);
}
